@@ -14,16 +14,20 @@
 
 void showErrorMsgBox(const char *fmt, ...)
 {
-	char strBuf[1024];
+	char strBuf[512+1];
 	va_list args;
 
 	// format the text string
 	va_start(args, fmt);
-	vsnprintf(strBuf, sizeof (strBuf), fmt, args);
+	vsnprintf(strBuf, sizeof (strBuf)-1, fmt, args);
 	va_end(args);
 
-	// window can be NULL here, no problem...
-	SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, "Critical Error", strBuf, video.window);
+	// SDL message boxes can be very buggy on Windows XP, use MessageBoxA() instead
+#ifdef _WIN32
+	MessageBoxA(NULL, strBuf, "Error", MB_OK | MB_ICONERROR);
+#else
+	SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, "Error", strBuf, NULL);
+#endif
 }
 
 void sanitizeFilenameChar(char *chr)
